from typing import Optional

class StringView:
    def __init__(self, buffer: str) -> None: ...
    @property
    def current(self) -> Optional[str]: ...
    @property
    def eof(self) -> bool: ...
    def undo(self) -> None: ...
    def skip_ws(self) -> bool: ...
    def skip_string(self, string: str) -> bool: ...
    def read_rest(self) -> str: ...
    def read(self, n: int) -> str: ...
    def get(self) -> str: ...
    def get_word(self) -> str: ...
    def get_quoted_word(self) -> Optional[str]: ...
