from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import Any, Tuple

def show_version() -> None: ...
def core(parser: Any, args: Any) -> None: ...

bot_template: str
gitignore_template: str
cog_template: str
cog_extras: str
translation_table: Any

def to_path(parser: Any, name: Any, *, replace_spaces: bool = ...) -> Path: ...
def newbot(parser: Any, args: Any) -> None: ...
def newcog(parser: Any, args: Any) -> None: ...
def add_newbot_args(subparser: Any) -> None: ...
def add_newcog_args(subparser: Any) -> None: ...
def parse_args() -> Tuple[ArgumentParser, Namespace]: ...
def main() -> None: ...
